/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.Map;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;
import org.jfree.data.xy.XYSeries;

public class BedHeightProcessor extends DefaultProcessor {

    private final static Logger log = Logger.getLogger(BedHeightProcessor.class);

    public static final String I18N_AXIS_LABEL = "chart.bedheight_middle.section.yaxis.label";

    public final static String FACET_BED_HEIGHTS_MIN = "bedheight_min";

    public final static String FACET_BED_HEIGHTS_MAX = "bedheight_min";

    protected static final double GAP_TOLERANCE = 0.101d;

    protected String yAxisLabel;

    @Override
    public void doOut(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {
        final XYSeries series = prepareSeries(bundle, theme, generator.getContext());
        if (series != null) {
            generator.addAxisSeries(series, this.axisName, visible);
        }
    }

    @Override
    public void doOut(final XYChartGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible, final int index) {
        final XYSeries series = prepareSeries(bundle, theme, generator.getContext());
        if (series != null) {
            generator.addAxisSeries(series, index, visible);
        }
    }

    /** Prepare an series, independent of axis. */
    private XYSeries prepareSeries(final ArtifactAndFacet bundle, final ThemeDocument theme, final CallContext context) {
        final Map<String, String> metaData = bundle.getFacet().getMetaData(bundle.getArtifact(), context);
        final StyledXYSeries series = new StyledXYSeries(bundle.getFacetName(), bundle.getFacetDescription(), theme);
        series.putMetaData(metaData, bundle.getArtifact(), context);
        this.yAxisLabel = metaData.get("Y");

        final Object raw = bundle.getData(context);
        if (raw == null) {
            return null;
        }
        if (!(raw instanceof double[][])) {
            log.error("Unkown datatype: " + raw.getClass().getName());
            return null;
        }

        final double[][] data = (double[][]) raw;
        StyledSeriesBuilder.addPoints(series, data, false, GAP_TOLERANCE);
        return series;
    }

    @Override
    public boolean canHandle(final String facetType) {
        return FacetTypes.BEDHEIGHT.equals(facetType) || FACET_BED_HEIGHTS_MIN.equals(facetType) || FACET_BED_HEIGHTS_MAX.equals(facetType)
                || FacetTypes.BED_DIFFERENCE_YEAR_HEIGHT1.equals(facetType) || FacetTypes.BED_DIFFERENCE_YEAR_HEIGHT2.equals(facetType)
                || FacetTypes.BED_DIFFERENCE_YEAR_HEIGHT1_FILTERED.equals(facetType) || FacetTypes.BED_DIFFERENCE_YEAR_HEIGHT2_FILTERED.equals(facetType);
    }

    @Override
    public String getAxisLabel(final DiagramGenerator generator) {
        final D4EArtifact flys = (D4EArtifact) generator.getMaster();
        final String unit = new RiverAccess(flys).getRiver().getWstUnit().getName();

        final CallMeta meta = generator.getContext().getMeta();

        if (this.yAxisLabel != null && !this.yAxisLabel.isEmpty()) {
            return Resources.getMsg(meta, this.yAxisLabel, new Object[] { unit });
        }
        return Resources.getMsg(meta, I18N_AXIS_LABEL, new Object[] { unit });
    }
}